package fr.velossity.sample.device.impl;

import org.osgi.framework.BundleActivator;
import org.osgi.framework.BundleContext;
import org.osgi.framework.Constants;
import org.osgi.framework.ServiceEvent;
import org.osgi.framework.ServiceListener;
import org.osgi.framework.ServiceReference;

import fr.velossity.sample.device.Device;

/**
 * 
 * @author C. Saint-Marcel
 * 
 */
public class Activator implements BundleActivator, ServiceListener {

	/**
	 * Keeps a reference on the bundle context.
	 */
	private BundleContext myContext;
	
	/**
	 * @see BundleActivator#start(BundleContext)
	 */
	public void start(BundleContext context) throws Exception {
		
		myContext = context;
		
		// Adds itself as a Registry listener for OSGi services
		context.addServiceListener(this, "(" + Constants.OBJECTCLASS + "=" + Device.class.getName() + ")");
		
		// Service class name
		String clazz = Device.class.getName();
		
		// REQUESTS a service implementing a Device service of type Mock 
		ServiceReference sr[] = context.getAllServiceReferences(clazz, "(type=Mock)");
		if(sr != null && sr.length != 0) {
			
			// BINDS to the service provider
			Device myDevice = (Device) context.getService(sr[0]);
			
			// INVOKES the service
			System.out.println("I found a mock device with identifer= " + myDevice.getIdentifier());
		
		} else {
			System.out.println("I found no mock device at that time");
		}
	}

	/**
	 * @see BundleActivator#stop(BundleContext)
	 */
	public void stop(BundleContext context) throws Exception {
	}

	/**
	 * NOTIFICATION for device services.
	 * OSGi callback to be notified of providers arrival and departure.
	 * @see ServiceListener#serviceChanged(ServiceEvent)
	 */
	public void serviceChanged(ServiceEvent event) {		
		ServiceReference sr = event.getServiceReference();
		// BINDS to the service provider
		Device myDevice = (Device) myContext.getService(sr);

		if(event.getType() == ServiceEvent.REGISTERED) {
			// INVOKES the service
			System.out.println("I found a mock device with identifer= " + myDevice.getIdentifier());
		}
		if(event.getType() == ServiceEvent.UNREGISTERING) {
			// INVOKES the service
			System.out.println("Mock device [" + myDevice.getIdentifier() + "] is unregistering");
		}
	}

}
